package com.example.springdemo.security;

import com.example.springdemo.security.jwt.JwtAuthenticationFilter;
import jakarta.annotation.Resource;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.config.Customizer;
import org.springframework.security.config.annotation.authentication.configuration.EnableGlobalAuthentication;
import org.springframework.security.config.annotation.method.configuration.EnableMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configurers.AbstractHttpConfigurer;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.AnonymousAuthenticationFilter;
import org.springframework.security.web.authentication.AuthenticationFilter;

@Configuration
@EnableWebSecurity // Enable Spring Security
@EnableGlobalAuthentication // Enable Spring Security's global authentication configuration
@EnableMethodSecurity(prePostEnabled = true) // Enable Spring Security's method security
public class SecurityFilterChainConfig {
    @Resource
    AuthenticationFilter authenticationFilter;

    @Bean
    public SecurityFilterChain SecurityFilterChain(@NotNull HttpSecurity http) throws Exception {
        var ignoreUrls = new String[]{"/login", "/logout", "/error"};
        var authedUrls = new String[]{"/users/*/**"};
        http
                .authorizeHttpRequests(
                        (request) -> request
                                .requestMatchers(authedUrls).authenticated() // authenticate all requests to authedUrls
                                .requestMatchers(ignoreUrls).permitAll() // permit all requests to ignoreUrls
                )
                .httpBasic(Customizer.withDefaults())
                .csrf(AbstractHttpConfigurer::disable)
                .sessionManagement(a -> a.sessionCreationPolicy(SessionCreationPolicy.STATELESS))
                .formLogin(AbstractHttpConfigurer::disable)
                .logout(AbstractHttpConfigurer::disable)
                .addFilterBefore(authenticationFilter, AnonymousAuthenticationFilter.class); // jwt filter;
        return http.build();
    }
}
